#!/usr/bin/env python3
from collections import namedtuple
import itertools
import copy
import requests


Pos = namedtuple("Pos", ["x", "y"])
Vel = namedtuple("Vel", ["vx", "vy"])


class Lander:
    landing_speed = 5
    height = 20
    width = 20

    def __init__(self, server_addr):
        self.server_addr = server_addr
        # Can be either None (not landed yet), False (failed landing)
        # or True (succesful landing)
        self.landed = None

    @classmethod
    def new_from_server(cls, server_addr="http://localhost:8000"):
        resp = requests.get(server_addr + "/start")
        lander = Lander(server_addr)
        lander.parse_server_resp(resp.text)
        return lander

    def parse_server_resp(self, resp):
        resp_lines = resp.split("\n")
        if len(resp_lines) == 1:
            if resp_lines[0] == "Krach!":
                self.landed = False
                return
            elif resp_lines[0] == "Zdar!":
                self.landed = True
                return
            else:
                assert(False)
        self.lid = float(resp_lines[0])
        posline_split = resp_lines[1].split(" ")
        self.pos = Pos(float(posline_split[0]), float(posline_split[1]))
        velline_split = resp_lines[2].split(" ")
        self.vel = Vel(float(velline_split[0]), float(velline_split[1]))
        self.fuel = float(resp_lines[3])
        lpline_split = resp_lines[4].split(" ")
        self.lp_l = float(lpline_split[0])
        self.lp_r = float(lpline_split[1])
        print("Lander resp: pos={}, vel={}".format(self.pos, self.vel))

    def send_movement_cmd(self, up=None, left=None, right=None):
        url = self.server_addr + "/stav?id={}".format(self.lid)
        if up is not None:
            url += "&nahoru={}".format(1 if up else 0)
        if left is not None:
            url += "&doleva={}".format(1 if left else 0)
        if right is not None:
            url += "&doprava={}".format(1 if right else 0)
        resp = requests.get(url)
        self.parse_server_resp(resp.text)

    def get_bounding_box(self):
        return (Pos(self.pos.x + 10, self.pos.y - 10),
                Pos(self.pos.y + 10, self.pos.y + 10))

    def after(self, secs, up=False, left=False, right=False):
        x = (self.pos.x + self.vel.vx*secs
             + (((6 if right else 0) - (6 if left else 0)) / 2 * secs))
        y = (self.pos.y + self.vel.vy*secs
             + ((-7 + (12 if up else 0)) / 2 * secs**2))
        vx = self.vel.vx + ((6 if right else 0) - (6 if left else 0)) * secs
        vy = self.vel.vy + (-7 + (12 if up else 0)) * secs
        new_lander = copy.copy(self)
        new_lander.pos = Pos(x, y)
        new_lander.vel = Vel(vx, vy)
        return new_lander

    def steps_vert_landing(self, debug=True, fancier=False):
        if not fancier:
            # First, let it fall, then engage thruster up at the last moment
            # Brute force approach.
            # i - seconds without thruster
            for i in itertools.count():
                i = i / 10
                nl = self.after(i)
                if debug: print("D1", i, nl.pos, nl.vel)

                last_y = nl.pos.y
                for j in itertools.count():
                    j = j / 10
                    nl2 = nl.after(j, up=True)
                    if debug: print("D2", j, nl2.pos, nl2.vel)
                    if nl2.pos.y > last_y:
                        break
                    last_y = nl2.pos.y
                    if nl2.pos.y <= 0:
                        if nl2.vel.vy >= -5:
                            return (i, j, nl2)
                        break

        # Fancier bruteforce approach. Still pretty awful.
        # And even more broken
        for i in range(100):
            i /= 10
            last_y = float("infinity")
            for j in range(100):
                j /= 10
                y = 2*self.vel.vy*i + self.pos.y - 3.5*i**2 - 7*i + 2.5*j**2
                vy = self.vel.vy - 7*i + 2.5*j
                if debug: print(i, j, y, vy, last_y)
                if y > last_y:
                    if debug: print("y>last_y")
                    break
                last_y = y
                if y <= 0 and vy >= -5:
                    if debug: print("SUCC!", i, j, y)


if __name__ == "__main__":
    l = Lander.new_from_server()
    print(l.steps_vert_landing(fancier=True, debug=True))
