#!/usr/bin/env python3
import sys

from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *

import game


class LanderGameWidget(QWidget):
    # How many llunits per 100px
    speed_scale = 40

    def __init__(self, server_addr, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.server_addr = server_addr
        self.start_game()

    def start_game(self):
        self.lander = game.Lander.new_from_server(self.server_addr)
        self.set_auto_scale()
        print(self.area_width, self.area_height)
        self.max_fuel = self.lander.fuel

        self.timer = QElapsedTimer()
        self.timer.start()

        self.update_timer = QTimer()
        # Set the time higher if the server is unresponsive (remote?)
        self.update_timer.setInterval(33)
        self.update_timer.timeout.connect(self.on_update_timer)
        self.update_timer.start()

        self.thrust_left = False
        self.thrust_right = False
        self.thrust_up = False

    def set_auto_scale(self):
#        lp_center_x = self.lander.lp_l + (self.lander.lp_r - self.lander.lp_l)/2
#        module_x_dist = abs(self.lander.pos.x - lp_center_x)
#        self.area_x_offset = abs(min(lp_center_x - module_x_dist, 0))
#        self.area_width = max(200, lp_center_x + module_x_dist*1.2 + self.area_x_offset)
#        self.area_height = self.area_width * (self.height() / self.width())
#        print("Scale:", self.area_width, self.area_height, self.area_x_offset)
#        return

        if self.lander.pos.x < 0:
            self.area_x_offset = -self.lander.pos.x
        else:
            self.area_x_offset = 0
        area_width = (max(self.lander.lp_l + self.lander.lp_r,
                          abs(self.lander.pos.x) * 1.2,
                          200)
                      + self.area_x_offset)
        area_height = max(self.lander.pos.y * 1.2, 200)
        area_height_by_width = area_width * (self.height() / self.width())
        area_width_by_height = (area_height * (self.width() / self.height())
                                + self.area_x_offset)
        if area_height > area_height_by_width:
            self.area_height = area_height
            self.area_width = area_width_by_height
        else:
            self.area_height = area_height_by_width
            self.area_width = area_width
        print("Scale:", self.area_width, self.area_height, self.area_x_offset)

    def lander_pos_x_to_px(self, x):
        return (self.width() / self.area_width) * (self.area_x_offset + x)

    def lander_pos_y_to_px(self, y):
        return (self.height() / self.area_height) * (self.area_height - y)

    def lander_scale_x_to_px(self, x):
        return (self.width() / self.area_width) * (x)

    def lander_scale_y_to_px(self, y):
        return (self.height() / self.area_height) * y

    def lander_pos_to_px(self, pos):
        return (self.lander_pos_x_to_px(pos.x),
                self.lander_pos_y_to_px(pos.y))

    def draw_lander(self, p):
        lander_box = self.lander.get_bounding_box()
        lx, ly = self.lander_pos_to_px(lander_box[0])
        lh = self.lander_scale_y_to_px(self.lander.height)
        lw = self.lander_scale_x_to_px(self.lander.width)
        #brush = QBrush(Qt.SolidPattern)
        #p.fillRect(lx, ly, lw, -lh, brush)
        p.drawPixmap(lx, ly - lh, QIcon("lunar_lander.svg").pixmap(lw, lh))

    def draw_landing_zone(self, p):
        lzx = self.lander_pos_x_to_px(self.lander.lp_l)
        lzy = self.lander_pos_x_to_px(self.lander.lp_r)
        p.drawLine(lzx, self.height() - 1, lzy, self.height() - 1)

    def draw_stats(self, p):
        # Fuel
        brush = QBrush(Qt.SolidPattern)
        p.drawRect(10, 10, 100, 20)
        p.fillRect(10, 10, (self.lander.fuel / self.max_fuel) * 100, 20, brush)

        # Vert. speed
        p.drawLine(60, 37, 60, 63)
        landing_speed_x = 60 - ((100 / self.speed_scale)
                                * self.lander.landing_speed)
        p.drawLine(landing_speed_x, 38, landing_speed_x, 62)
        p.fillRect(60, 40, (self.lander.vel.vy / self.speed_scale) * 100,
                   20, brush)

        # Horiz. speed
        p.drawLine(60, 67, 60, 93)
        p.drawLine(landing_speed_x, 68, landing_speed_x, 92)
        landing_speed_x2 = 60 + ((100 / self.speed_scale)
                                 * self.lander.landing_speed)
        p.drawLine(landing_speed_x2, 68, landing_speed_x2, 92)
        p.fillRect(60, 70, (self.lander.vel.vx / self.speed_scale) * 100,
                   20, brush)

    def paintEvent(self, e):
        p = QPainter(self)
        self.draw_lander(p)
        self.draw_landing_zone(p)
        self.draw_stats(p)

        if self.lander.landed is not None:
            text = ("Landing succesful" if self.lander.landed
                    else "Landing failed")
            text += "\nTo try again, press \"R\""
            p.drawText(QRect(0, 0, self.width(), self.height()),
                       Qt.AlignCenter, text)

    def on_update_timer(self):
        print("Update: up={}, left={}, right={}".format(self.thrust_up,
                                                        self.thrust_left,
                                                        self.thrust_right))
        self.lander.send_movement_cmd(up=self.thrust_up, left=self.thrust_left,
                                      right=self.thrust_right)
        self.set_auto_scale()
        self.repaint()
        if self.lander.landed is not None:
            self.update_timer.stop()

    def keyPressEvent(self, e):
        if e.key() == Qt.Key_Up:
            self.thrust_up = True
        elif e.key() == Qt.Key_Left:
            self.thrust_left = True
        elif e.key() == Qt.Key_Right:
            self.thrust_right = True
        elif e.key() == Qt.Key_R:
            self.start_game()

    def keyReleaseEvent(self, e):
        if e.key() == Qt.Key_Up:
            self.thrust_up = False
        elif e.key() == Qt.Key_Left:
            self.thrust_left = False
        elif e.key() == Qt.Key_Right:
            self.thrust_right = False

    def mouseMoveEvent(self, e):
        if e.buttons() != Qt.LeftButton:
            return
        lx, ly = self.lander_pos_to_px(self.lander.pos)
        print("DM", lx, ly, e.x(), e.y())
        if e.x() - lx > 20:
            print("Mouse: right")
            self.thrust_right = True
            self.thrust_left = False
        if e.x() - lx < -20:
            print("Mouse: left")
            self.thrust_left = True
            self.thrust_right = False
        if e.y() - ly < -20:
            print("Mouse: up")
            self.thrust_up = True

    def mouseReleaseEvent(self, e):
        self.thrust_right = False
        self.thrust_left = False
        self.thrust_up = False


if __name__ == "__main__":
    if len(sys.argv) == 2:
        server_addr = sys.argv[1]
    else:
        server_addr = "http://localhost:8000"
    print(server_addr)
    if server_addr[:7] != "http://":
        server_addr = "http://" + server_addr
    app = QApplication(sys.argv)
    lgw = LanderGameWidget(server_addr)
    lgw.show()
    sys.exit(app.exec())
